/* ============================================================
 *
 * This file is a part of digiKam project
 * https://www.digikam.org
 *
 * Date        : 2009-08-03
 * Description : digiKam export headers
 *
 * NOTE: Only use pure C comments here as this header is shared between C and C++
 *
 * Copyright (C) 2009-2022 by Gilles Caulier <caulier dot gilles at gmail dot com>
 *
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General
 * Public License as published by the Free Software Foundation;
 * either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * ============================================================ */

#ifndef DIGIKAM_EXPORT_HEADERS_H
#define DIGIKAM_EXPORT_HEADERS_H

/**
 * These header files automatically generated by generate_export_header() macro
 * for each shared library targets.
 */
#include "digikam_core_export.h"
#include "digikam_database_export.h"
#include "digikam_gui_export.h"

#endif /* DIGIKAM_EXPORT_HEADERS_H */
